class PhaseConfig {
  final String _phase;

  PhaseConfig._dev() : _phase = 'alpha';

  PhaseConfig._staging() : _phase = 'beta';

  PhaseConfig._live() : _phase = 'v1';

  factory PhaseConfig(Phase phase) {
    switch (phase) {
      case Phase.dev:
        {
          _instance = PhaseConfig._dev();
          break;
        }
      case Phase.staging:
        {
          _instance = PhaseConfig._staging();
          break;
        }
      case Phase.live:
        {
          _instance = PhaseConfig._live();
          break;
        }
    }
    return _instance;
  }

  static PhaseConfig _instance = PhaseConfig._staging();

  static String get phase => _instance._phase;

  static String get paymentWidgetHtml => '''
    <html lang="ko">
    
    <head>
        <meta charset="UTF-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title></title>
        <script type='text/javascript'>
          window.PaymentWidgetFlutterSDK = {
            message : {
              postMessage : function(_message) {
                window.flutter_inappwebview.callHandler('PaymentWidgetFlutterSDK', _message);
              }
            },
            error : {
              postMessage : function(error) {
                window.flutter_inappwebview.callHandler('PaymentWidgetFlutterSDK', JSON.stringify({
                  name: 'error', 
                  params: JSON.parse(error)
                }));
              }
            }
          }
        </script>
        <script src="https://js.tosspayments.com/${_instance._phase}/payment-widget"></script>
    </head>
    
    <body style="margin:0;padding:0">
    <div id="payment-method"></div>
    <div id="agreement"></div>
    </body>
    </html>
  ''';
}

enum Phase {
  dev,
  staging,
  live,
}
