import 'dart:convert';

import 'package:tosspayments_widget_sdk_flutter/listener/agreement_status_listener.dart';
import 'package:tosspayments_widget_sdk_flutter/payment_widget.dart';
import 'package:tosspayments_widget_sdk_flutter/widgets/widget_container.dart';

import '../model/agreement_status.dart';
import '../model/payment_widget_options.dart';
import '../webview/javascript_channel.dart';

class AgreementWidget extends WidgetContainer {
  AgreementWidget({required PaymentWidget paymentWidget, required String selector})
      : super(key: paymentWidget.getGlobalKey<PaymentAgreementWidgetState>(selector), paymentWidget: paymentWidget);

  @override
  WidgetContainerState createState() => PaymentAgreementWidgetState();
}

class PaymentAgreementWidgetState extends WidgetContainerState {
  AgreementStatusListener? agreementStatusListener;

  Future<AgreementWidgetControl> renderAgreement({RenderAgreementOptions? options}) async {
    addJavascriptChannels(agreementWidgetJavascriptChannels);

    final optionsJson = jsonEncode(options?.toJson() ?? '');

    String renderScript = 'const agreementWidget = paymentWidget.renderAgreement(\'#agreement\', $optionsJson);';
    try {
      await renderWidget(renderScript: renderScript);
      return AgreementWidgetControl._(
          addAgreementStatusListener: (listener) {
            agreementStatusListener = listener;
          },
          getAgreementStatus: _getAgreementStatus);
    } catch (fail) {
      return Future.error(fail);
    }
  }

  Future<AgreementStatus> _getAgreementStatus() async {
    return AgreementStatus.fromJson(await evaluateJavascriptWithResolve('agreementWidget.getAgreementStatus()'));
  }

  Set<JavascriptChannel> get agreementWidgetJavascriptChannels => {
        JavascriptChannel(
            name: "updateAgreementStatus",
            onReceived: (jsonObject) {
              var agreementStatus = AgreementStatus.fromJson(jsonObject);
              agreementStatusListener?.onAgreementStatusChanged(agreementStatus);
            }),
      };
}

/// [renderAgreement]로 얻을 수 있는 클래스입니다.
/// [getAgreementStatus] : 고객의 필수 약관 동의 상태를 반환합니다.
/// [addAgreementStatusListener] : 약관 동의 상태 리스너를 추가합니다.
class AgreementWidgetControl {
  final void Function(AgreementStatusListener) addAgreementStatusListener;
  final Future<AgreementStatus> Function() getAgreementStatus;

  const AgreementWidgetControl._({
    required this.addAgreementStatusListener,
    required this.getAgreementStatus,
  });
}
